@@ -2,7 +2,7 @@
 
   CWindow.cpp
 
-  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -658,9 +658,25 @@ BEGIN_METHOD_VOID(Window_Raise)
 
 END_METHOD
 
+static bool check_opened(CWINDOW *_object, bool modal)
+{
+	if (THIS->toplevel && THIS->opened)
+	{
+		if (modal || WINDOW->isModal())
+		{
+			GB.Error("Window is already opened");
+			return TRUE;
+		}
+	}
+	
+	return FALSE;
+}
 
 BEGIN_METHOD_VOID(Window_Show)
 
+	if (check_opened(THIS, FALSE))
+		return;
+
 	if (emit_open_event(THIS))
 		return;
 
@@ -701,6 +717,9 @@ END_METHOD
 
 BEGIN_METHOD_VOID(Window_ShowModal)
 
+	if (check_opened(THIS, TRUE))
+		return;
+
 	THIS->ret = 0;
 
 	if (!emit_open_event(THIS))
@@ -722,6 +741,9 @@ BEGIN_METHOD(Window_ShowPopup, GB_INTEGER x; GB_INTEGER y)
 
 	QPoint pos;
 
+	if (check_opened(THIS, TRUE))
+		return;
+
 	if (MISSING(x) || MISSING(y))
 		pos = QCursor::pos();
 	else
@@ -1650,9 +1672,10 @@ void MyMainWindow::activateLater()
 
 void MyMainWindow::present(QWidget *parent)
 {
-	//CWIDGET *_object = CWidget::get(this);
-	//CWIDGET *_parent = parent ? CWidget::get(parent) : 0;
-	//qDebug("present: %p %s: parent = %p %s", THIS, _object->name, _parent, _parent ? _parent->name : "");
+	/*CWIDGET *_object = CWidget::get(this);
+	CWIDGET *_parent = parent ? CWidget::get(parent) : 0;
+	
+	qDebug("present: %p %s: parent = %p %s", THIS, _object->name, _parent, _parent ? _parent->name : "");*/
 
 	if (parent)
 		_screen = QApplication::desktop()->screenNumber(parent);
@@ -1815,7 +1838,11 @@ void MyMainWindow::showModal(void)
 
 	parent = CWINDOW_Current;
 	if (!parent)
+	{
 		parent = CWINDOW_Main;
+		if (!parent)
+			parent = CWINDOW_Active;
+	}
 
 	present(parent ? CWidget::getTopLevel((CWIDGET *)parent)->widget.widget : 0);
 	setEventLoop();
@@ -2498,7 +2525,6 @@ void MyMainWindow::configure()
 			menuBar->move(0, -menuBar->height());
 			menuBar->lower();
 		}
-		//qDebug("configure: %s (%d %d)", GB.GetClassName(THIS), this->width(), this->height());
 
 		geom = QRect(0, 0, this->width(), this->height());
 
@@ -2512,7 +2538,10 @@ void MyMainWindow::configure()
 	}
 
 	if (arrange)
+	{
 		CCONTAINER_arrange(THIS);
+		CMENU_update_menubar(THIS);
+	}
 
 	//qDebug(">>> THIS->menuBar = %p  menuBar() = %p", THIS->menuBar, menuBar());
 
@@ -2737,7 +2766,7 @@ bool CWindow::eventFilter(QObject *o, QEvent *e)
 		{
 			MyMainWindow *w = (MyMainWindow *)o;
 
-			if (THIS->toplevel && !THIS->popup && (!THIS->moved || w->isModal()))
+			if (THIS->toplevel && !THIS->popup && !THIS->moved)
 				w->center();
 
 			//handle_focus(THIS);
@@ -2853,15 +2882,28 @@ CMENU *CWindow::findMenu(CWINDOW *_object, const char *name)
 {
 	int i;
 	CMENU *menu;
+	CWIDGET *parent;
 
-	if (THIS->menuBar)
+	for(;;)
 	{
-		for (i = 0; i < THIS->menuBar->actions().count(); i++)
+		if (THIS->menuBar)
 		{
-			menu = CMenu::dict[THIS->menuBar->actions().at(i)];
-			if (menu && !strcasecmp(menu->widget.name, name))
-				return menu;
+			for (i = 0; i < THIS->menuBar->actions().count(); i++)
+			{
+				menu = CMenu::dict[THIS->menuBar->actions().at(i)];
+				if (!menu)
+					continue;
+				if (!strcasecmp(menu->widget.name, name))
+					return menu;
+			}
 		}
+		
+		parent = (CWIDGET *)CWIDGET_get_parent(THIS);
+		if (!parent)
+			break;
+		_object = CWidget::getWindow(parent);
+		if (!THIS)
+			break;
 	}
 
 	return NULL;
